package integration

import (
	"fmt"
	"testing"

	"github.com/chromedp/chromedp"
	"github.com/stretchr/testify/require"
	"github.com/tofutf/tofutf/internal"
	"github.com/tofutf/tofutf/internal/team"
)

// TestWeb is a random walkthrough of the Web UI
func TestWeb(t *testing.T) {
	integrationTest(t)

	daemon, org, ctx := setup(t, nil)
	user := userFromContext(t, ctx)

	team, err := daemon.Teams.Create(ctx, org.Name, team.CreateTeamOptions{
		Name: internal.String("devops"),
	})
	require.NoError(t, err)
	err = daemon.Users.AddTeamMembership(ctx, team.ID, []string{user.Username})
	require.NoError(t, err)

	browser.Run(t, ctx, chromedp.Tasks{
		// create workspace
		createWorkspace(t, daemon.System.Hostname(), org.Name, "my-workspace"),
		// assign workspace manager role to devops team
		chromedp.Tasks{
			// go to org
			chromedp.Navigate(organizationURL(daemon.System.Hostname(), org.Name)),
			screenshot(t),
			// list teams
			chromedp.Click("#teams > a", chromedp.ByQuery),
			screenshot(t),
			// select devops team
			chromedp.Click("#item-team-devops", chromedp.ByQuery),
			screenshot(t),
			// tick checkbox for workspace manager role
			chromedp.Click("#manage_workspaces", chromedp.ByQuery),
			// submit form
			chromedp.Submit("#manage_workspaces", chromedp.NodeVisible, chromedp.ByQuery),
			screenshot(t, "team_permissions_added_workspace_manager"),
			// confirm permissions updated
			matchText(t, "//div[@role='alert']", "team permissions updated"),
		},
		// add write permission on workspace to devops team
		addWorkspacePermission(t, daemon.System.Hostname(), org.Name, "my-workspace", team.ID, "write"),
		// list users
		chromedp.Tasks{
			// go to org
			chromedp.Navigate(organizationURL(daemon.System.Hostname(), org.Name)),
			screenshot(t),
			// list users
			chromedp.Click("#users > a", chromedp.ByQuery),
			screenshot(t),
			matchText(t, fmt.Sprintf("#item-user-%s #username", user.Username), user.Username, chromedp.ByQuery),
		},
		// list team members
		chromedp.Tasks{
			// go to org
			chromedp.Navigate(organizationURL(daemon.System.Hostname(), org.Name)),
			screenshot(t),
			// list teams
			chromedp.Click("#teams > a", chromedp.ByQuery),
			// select owners team
			chromedp.Click("#item-team-owners", chromedp.ByQuery),
			screenshot(t),
			matchText(t, fmt.Sprintf("#item-user-%s #username", user.Username), user.Username, chromedp.ByQuery),
		},
	})
}
